/*
 * Copyright (C) 2002 Edscott Wilson Garcia
 * EMail: edscott@imp.mx
 *
 * Some code in here contributed by the FSF
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <grp.h>
#include <pwd.h>
#include <regex.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "glade_gui.h"
#include "glade_support.h"
#include "glade_callbacks.h"

#include "constants.h"
#include "types.h"

#include "filter.h"
#include "misc.h"

static GList *filter_list = NULL;


char *get_filter(GtkWidget * window)
{
    GtkWidget *filter_entry, *filter_combo;
    char *filter;
    char *f;
    GList *ee = filter_list;

    filter_entry = lookup_widget(window, "combo_entry2");
    filter_combo = lookup_widget(window, "filter_combo");
    if(!GTK_WIDGET_VISIBLE(lookup_widget(window, "filter_box")))
    {
	return "*";
    }
    if(!filter_entry)
	g_assert_not_reached();

    f = (char *)gtk_entry_get_text(GTK_ENTRY(filter_entry));
    while(ee)
    {
	if(strcmp(f, (char *)ee->data) == 0)
	    break;
	ee = ee->next;
    }
    if(!ee)
    {
	filter = g_strdup(f);
	filter_list = g_list_prepend(filter_list, filter);
	gtk_combo_set_popdown_strings(GTK_COMBO(filter_combo), filter_list);
    }
    else
    {
	filter = (char *)ee->data;
    }

    if(!filter || strlen(filter) == 0)
	return "*";
    return filter;
}

char *filter2regex(tree_entry_t * en)
{
    char *regex;
    char *filter = en->filter;
    /* change stars to regexp */
    if(!filter || strcmp(filter, "*") == 0 || strlen(filter) == 0)
    {
	regex = g_strdup("^.+$");
    }
    else
    {
	/*Let's translate stars to regexp */
	regex = (char *)malloc(2 * strlen(filter) + 1 + strlen("|^\\..+"));
	if(filter[0] == '*' && filter[strlen(filter) - 1] == '*')
	{
	    strcpy(regex, filter + 1);
	    regex[strlen(regex) - 1] = 0;
	}
	else if(filter[0] == '*')
	{
	    strcpy(regex, filter + 1);
	    strcat(regex, "$");
	    if(SHOWS_HIDDEN(en->type))
	    {
		strcat(regex, "|\\.");
		strcat(regex, filter + 1);
		strcat(regex, "$");
	    }
	}
	else if(filter[strlen(filter) - 1] == '*')
	{
	    strcpy(regex + 1, filter);
	    regex[0] = '^';
	    regex[strlen(filter)] = 0;
	    if(SHOWS_HIDDEN(en->type))
	    {
		strcat(regex, "|^\\.");
		strcat(regex, filter);
		regex[strlen(regex) - 1] = 0;
	    }
	}
	else if(strchr(filter, '*'))
	{
	    strcpy(regex + 1, filter);
	    regex[0] = '^';
	    *strchr(regex, '*') = 0;
	    strcat(regex, ".+");
	    strcat(regex, strchr(filter, '*') + 1);
	    strcat(regex, "$");
	    if(SHOWS_HIDDEN(en->type))
	    {
		char *aux;
		aux = g_strdup(regex + 1);
		strcat(regex, "|^\\.");
		strcat(regex, aux);
		g_free(aux);
	    }
	}
	else
	    strcpy(regex, filter);

    }
    /*printf("DBG:regex=%s\n",regex); */
    return regex;
}

/* OjO return value must be freed */
regex_t *get_regex_filter(GtkTreeView * treeview, tree_entry_t * en)
{
    static regex_t preg;
    char *regex;
    tree_details_t *tree_details = get_tree_details(treeview);
    char *filter = get_filter(tree_details->window);

    if(en->filter)
	g_free(en->filter);
    if(filter)
	en->filter = g_strdup(filter);
    else
	en->filter = g_strdup("*");
    regex = filter2regex(en);

    regcomp(&preg, regex, REG_EXTENDED | REG_ICASE | REG_NOSUB);
    g_free(regex);
    return &preg;
}

/* callbacks */

void on_filter_activate(GtkEntry * entry, gpointer user_data)
{
    on_refresh((GtkButton *) entry, user_data);
}

void on_filter_clear(GtkButton * button, gpointer user_data)
{
    GtkEntry *entry;
    GtkTreeView *treeview = get_treeview((GtkWidget *) button);
    tree_details_t *tree_details = (tree_details_t *) get_tree_details(treeview);
    entry = (GtkEntry *) lookup_widget(tree_details->window, "combo_entry2");
    gtk_entry_set_text(entry, "");
}
